/**
 * Copyright (c) 2025 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_CRSHR_HPP_
#define _HEAVY_CONTEXT_WSTD_CRSHR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvMath.h"
#include "HvControlVar.h"
#include "HvControlCast.h"
#include "HvSignalLine.h"
#include "HvControlBinop.h"
#include "HvSignalVar.h"

class Heavy_WSTD_CRSHR : public HeavyContext {

 public:
  Heavy_WSTD_CRSHR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_CRSHR();

  const char *getName() override { return "WSTD_CRSHR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        CRSHR = 0xE2DA4DC3, // Crshr
        MIX = 0x97D46AC6, // Mix
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_Qix8xFMr_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_cXDk8esb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_IBRuQTTM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_ZUqaCYae_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_niLxgwlU_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_Qw7JdIlJ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_84ug1GoH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_PAdubZFv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_sAtWGd7G_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Tym3RrLo_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_n9zCXVqZ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_TM5aSZcJ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_QKS0VChP_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_oA6PECpR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_8pztFTYL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_o7dmYQhE_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_vZTpVoOB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_bV2QPLts_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_MrW6GNec_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_cM1MhVWg_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_QuM3XZSN_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_t5ImBiA9_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_7wwtLUBg_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_7r8IPnSy_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_SyxOzLjC_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_m4HA5G6B_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_3AVKTDL7;
  SignalLine sLine_S2ZtcKX8;
  SignalLine sLine_mnF1YaCa;
  ControlBinop cBinop_Qix8xFMr;
  ControlBinop cBinop_cXDk8esb;
  ControlBinop cBinop_IBRuQTTM;
  ControlBinop cBinop_ZUqaCYae;
  ControlBinop cBinop_84ug1GoH;
  ControlBinop cBinop_sAtWGd7G;
  ControlBinop cBinop_Tym3RrLo;
  ControlBinop cBinop_n9zCXVqZ;
  ControlVar cVar_TM5aSZcJ;
  ControlVar cVar_QKS0VChP;
  SignalVarf sVarf_0j81eluf;
  SignalVarf sVarf_nNP8HeM1;
  SignalVarf sVarf_Bydpy8yP;
  SignalVarf sVarf_1R4Ow0kf;
  ControlBinop cBinop_bV2QPLts;
  ControlBinop cBinop_t5ImBiA9;
};

#endif // _HEAVY_CONTEXT_WSTD_CRSHR_HPP_
